/*
  Copyright 2023 ipydrawio contributors

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/
/* eslint-disable */
/**
 * This file was automatically generated by json-schema-to-typescript.
 * DO NOT MODIFY IT BY HAND. Instead, modify the source JSONSchema file,
 * and run json-schema-to-typescript to regenerate this file.
 */

/**
 * An array of font family names to be added before defaultFonts
 */
export type FontSet = (
  | string
  | {
      fontFamily?: string;
      fontUrl?: string;
      [k: string]: unknown;
    }
)[];
export type Color = string;
/**
 * Colour codes to be added before presetColors (no leading # for the colour codes)
 */
export type ColorSet = Color[];
/**
 * Colour codes for the lower palette in the colour dialog (no leading # for the colour codes).
 */
export type ColorSet1 = Color[];
/**
 * Defines an array of IDs to load custom libraries.
 */
export type LibraryIdList = string[];
/**
 * An array of font family names in the format panel font drop-down list. All fonts must be installed on the server and all clients or be added using the fontCss option.
 */
export type FontSet1 = (
  | string
  | {
      fontFamily?: string;
      fontUrl?: string;
      [k: string]: unknown;
    }
)[];
/**
 * Defines an array of strings of library keys which will be available in the More Shapes dialog. If you define this as null, all libraries will be visible. If you leave the array empty, no libraries will be visible
 */
export type LibraryKeyList = string[];
/**
 * Defines an array of objects that list additional libraries and sections in the left panel and the More Shapes dialog
 */
export type LibraryDefList = LibrarySection[];
/**
 * Defines an array of plugin URLs that should be loaded with the diagram editor. Plugins are JavaScript files that can modify the editor UI and behaviour.
 */
export type PluginUrlList = string[];
/**
 * Colour codes for the upper palette in the colour dialog (no leading # for the colour codes).
 */
export type ColorSet2 = Color[];
export type Off = '0';
export type On = '';
export type On1 = '';
export type Off1 = '0';
export type On2 = '1';
export type Off2 = '';
/**
 * [min|atlas|dark|sketch]: Uses the Minimal, Atlas, Dark or Sketch UI theme (default is Kennedy).
 */
export type UITheme = 'min' | 'atlas' | 'dark' | 'kennedy' | 'sketch';

/**
 * Configure Diagram Settings
 */
export interface DiagramDocuments {
  /**
   * disable returning focus to the main frame when the `escape` key is pressed
   */
  disableEscapeFocus?: boolean;
  drawioConfig?: DrawioConfiguration;
  drawioExportUrl?: DrawioExportURL;
  drawioUrlParams?: DrawioURLParams;
  [k: string]: unknown;
}
/**
 * Configuration options sent to each drawio editor. https://www.diagrams.net/doc/faq/configure-diagram-editor
 */
export interface DrawioConfiguration {
  /**
   * Defines the delay (in ms) between the last change and the autosave of the file
   */
  autosaveDelay?: number;
  /**
   * Specifies if the XML output should be compressed.
   */
  compressXml?: boolean;
  /**
   * Defines a string with CSS rules to be used to configure the diagrams.net user interface.
   */
  css?: string;
  /**
   * TBD: see https://www.diagrams.net/doc/faq/configure-diagram-editor
   */
  customColorSchemes?: {
    [k: string]: unknown;
  };
  customFonts?: FontSet;
  customPresetColors?: ColorSet;
  /**
   * Set the server endpoint region. The default is to use your closest region, (either EU or US). If lockdown is set to true, dataGovernance is ignored.
   */
  dataGovernance?: string;
  defaultColorSchemes?: ColorScheme;
  defaultColors?: ColorSet1;
  defaultCustomLibraries?: LibraryIdList;
  /**
   * Defines the default length for new connectors
   */
  defaultEdgeLength?: number;
  /**
   * TBD: see https://www.diagrams.net/doc/faq/configure-diagram-editor
   */
  defaultEdgeStyle?: {
    [k: string]: unknown;
  };
  defaultFonts?: FontSet1;
  /**
   * Defines a semicolon-separated list of library keys (unique names) in a string to be initially displayed in the left panel (e.g. general;uml;company-graphics). Possible keys include custom entry IDs from the libraries field, or keys for the libs URL parameter
   */
  defaultLibraries?: string;
  /**
   * TBD: see https://www.diagrams.net/doc/faq/configure-diagram-editor
   */
  defaultVertexStyle?: {
    [k: string]: unknown;
  };
  /**
   * Defines the XML for blank diagrams
   */
  emptyDiagramXml?: string;
  /**
   * Defines the XML for blank diagrams and libraries
   */
  emptyLibraryXml?: string;
  /**
   * Specifies if the open and new library functions are enabled (true or false, the default true).
   */
  enableCustomLibraries?: boolean;
  enabledLibraries?: LibraryKeyList;
  /**
   * Defines a string with CSS rules for web fonts to be used in diagrams. This should be one or more @font-face rule,
   */
  fontCss?: string;
  /**
   * JSON structure with key, value pairs to define global variables for system-wide placeholders. You should keep the number of entries small.
   */
  globalVars?: {
    [k: string]: unknown;
  };
  /**
   * Defines the number of minor grid steps (14.3.2 and later).
   */
  gridSteps?: number;
  libraries?: LibraryDefList;
  /**
   * Disable data transmission, apart from directly between your browser and your selected data storage location. Default is false.
   */
  lockdown?: boolean;
  /**
   * Defines the maximum size for images in bytes. Default is 1000000.
   */
  maxImageBytes?: number;
  /**
   * Defines the maximum width or height of the image, where the lowest value is used.
   */
  maxImageSize?: number;
  /**
   * Ignores the current settings on the client-side if this is set to true
   */
  override?: boolean;
  plugins?: PluginUrlList;
  presetColors?: ColorSet2;
  /**
   * Disables word wrap and complex formatting for labels by default to avoid foreignObjects in the SVG output (14.5.9 and later).
   */
  simpleLabels?: boolean;
  styles?: Style[];
  /**
   * Defines the URL of the source file for the Templates dialog (multiple <template> tags are allowed).
   */
  templateFile?: string;
  /**
   * Defines the height for the entries in the left panel
   */
  thumbHeight?: number;
  /**
   * Defines the width for the entries in the left panel
   */
  thumbWidth?: number;
  /**
   * The current version of the configuration (any string, e.g. 1.0). If this is different from the last used version, then the current settings on the client-side (.drawio-config) will be reset. The default is null. Change this to force the stored settings in the client to be reset and apply the current configuration
   */
  version?: string;
  /**
   * Defines the zoom factor for mouse wheel and trackpad zoom. Default is 1.2. (14.7.0 and later).
   */
  zoomFactor?: number;
  [k: string]: unknown;
}
/**
 * TBD: see https://www.diagrams.net/doc/faq/configure-diagram-editor
 */
export interface ColorScheme {
  fill?: Color;
  font?: string;
  gradient?: Color;
  stroke?: Color;
  [k: string]: unknown;
}
export interface LibrarySection {
  entries?: LibraryEntry[];
  title?: LibraryResource;
  [k: string]: unknown;
}
export interface LibraryEntry {
  desc?: LibraryResource;
  id?: string;
  libs?: DrawioLibrary[];
  preview?: string;
  title?: LibraryResource;
  [k: string]: unknown;
}
export interface LibraryResource {
  main?: string;
  [k: string]: unknown;
}
export interface DrawioLibrary {
  data?: {
    aspect?: string;
    h?: number;
    w?: number;
    /**
     * NB: appears to be base64-encoded
     */
    xml?: string;
    [k: string]: unknown;
  };
  tags?: string;
  title?: LibraryResource;
  url?: string;
  [k: string]: unknown;
}
export interface Style {
  commonStyle?: {
    [k: string]: unknown;
  };
  edgeStyle?: {
    [k: string]: unknown;
  };
  graph?: {
    [k: string]: unknown;
  };
  [k: string]: unknown;
}
/**
 * URL parameters for the export URL.
 */
export interface DrawioExportURL {
  params?: {
    /**
     * Whether all pages should be rendered 0 0 1
     */
    allPages?: string;
    /**
     * Whether the response data should be base64 (and png embedded data) encoded 0 or 1
     */
    base64?: string;
    /**
     * The background colour for the image #ff8888
     */
    bg?: string;
    /**
     * The size of the border for the page 0 10
     */
    border?: number;
    /**
     * The data to embed into the png -
     */
    data?: string;
    /**
     * The key to use when embedding the data into the png myKey
     */
    dataHeader?: string;
    /**
     * Embed data in the png with the dataHeader key null 0 or 1
     */
    embedData?: string;
    /**
     * Embed the diagram data in the png 0 or 1
     */
    embedXml?: string;
    /**
     * Additional detail how what should be rendered, e.g. layer-names
     */
    extras?: {
      [k: string]: unknown;
    };
    /**
     * The filename to included in the content-disposition header myFile.png
     */
    filename?: string;
    /**
     * The renderering format for the diagram png pdf
     */
    format?: string;
    /**
     * The index of the page to start rendering from 1
     */
    from?: number;
    /**
     * Set the view port width 0 10
     */
    h?: number;
    /**
     * The id of the page to render page id
     */
    pageId?: string;
    /**
     * The scale for the rendered diagram 1 1.5
     */
    scale?: number;
    /**
     * The index of the page to finish rendering at 2
     */
    to?: number;
    /**
     * Set the view port height 0 10
     */
    w?: number;
    [k: string]: unknown;
  };
  /**
   * URL for (local) drawio-export server https://github.com/jgraph/draw-image-export2
   */
  url?: string;
  [k: string]: unknown;
}
/**
 * URL parameters for the drawio iframe https://web.archive.org/web/20210425055302/https://www.diagrams.net/doc/faq/supported-url-parameters
 */
export interface DrawioURLParams {
  /**
   * url: Sets the base URL to use for links (use this if `document.referrer` is not available in embed mode).
   */
  base?: string;
  /**
   * 60: Sets the border width used in lightbox mode (the default is `60`).
   */
  border?: number;
  /**
   * 0/1: Disables local storage as a storage location (`0`) or shows the browser option in the storage dialog (`1`).
   */
  browser?: (Off | On) & string;
  /**
   * 0: Uses the chromeless read-only viewer.
   */
  chrome?: (On1 | Off1) & string;
  /**
   * key1;key2;key;...;keyN: Specifies custom libraries (keys are file IDs or URLs with a U-prefix)
   */
  clibs?: string;
  /**
   * 1: Runs diagrams.net in client mode. This displays a normal UI and sends a `ready` message to the opener or parent when the page is loaded. After receiving a message containing XML or compressed XML, a local file is created. The file is then set to `modified` so the connection to the window can be closed after sending the initial XML. Whether the XML is loaded again after a page refresh, depends on the caller implementation. Use `proto=json` to use a custom protocol for special cases.
   */
  client?: (On2 | Off2) & string;
  /**
   * 1: Shows the *Close* button in chromeless mode which closes the window when selected.
   */
  close?: (On2 | Off2) & string;
  /**
   * 1: Sends the `configure` event and waits for the `configure` action. See more about the embed mode. **NEW `14.6.10`**
   */
  configure?: (On1 | Off1) & string;
  /**
   * regex: Checks whether URLS have CORS enabled using a URL encoded regular expression, e.g.cors=%5Ehttps%3F%3A%5C%2F%5C%2F.*%5C.github%5C.io%5C%2F **TBD `14.6.10`**
   */
  cors?: string;
  /**
   * url/name: Creates a new file from a template URL. If the value is not a URL and is not empty, the script will try to use `window.opener_url`. In embed mode, `window.opener[name]` will be used to get the initial XML. Note that this requires the same origin policy in the opener/parent for reading the variable.
   */
  create?: string;
  /**
   * [0|1]: Disables/enables dark mode in Sketch and Minimal editor themes.
   */
  dark?: (Off | On) & string;
  /**
   * data: Gets the XML data for the diagram (used in client mode to bypass message passing and cross frame access). **TBD `14.6.10`**
   */
  data?: string;
  /**
   * 0: Disables the Dropbox integration.
   */
  db?: (On1 | Off1) & string;
  /**
   * 1: Shortcut for `db=0&gapi=0&math=0&picker=0`, disables the splash screen and creates an empty, local diagram file.
   */
  demo?: (On2 | Off2) & string;
  /**
   * {compressed-json}: Passes the CSV import descriptor example. Works with the www.draw.io domain only. **NEW `14.6.10`**
   */
  desc?: {
    [k: string]: unknown;
  };
  /**
   * 0: Disables draft states (does not save unsaved files in *IndexedDB*).
   */
  drafts?: (On1 | Off1) & string;
  /**
   * 0: Simulates app.diagrams.net regardless of the domain name (uses the old app ID).
   */
  drive?: (On1 | Off1) & string;
  /**
   * move: Disables the handling of connector segments without having previously selected the connector.
   */
  edge?: string;
  /**
   * `edit=url`: Adds a link for the *Edit* button in chromeless mode (use `edit=_blank` to edit the diagram as a new copy).
   */
  edit?: string | '_blank';
  /**
   * `embed=1:` Runs diagrams.net in embed mode. Use this mode with embed.diagrams.net only. **NEW `14.6.10`**
   */
  embed?: {
    [k: string]: unknown;
  };
  /**
   * 0: Disables the format panel on the right.
   */
  format?: (On1 | Off1) & string;
  /**
   * 0: Disables the Google integration.
   */
  gapi?: (On1 | Off1) & string;
  /**
   * 0: Disables the Github integration.
   */
  gh?: (On1 | Off1) & string;
  /**
   * url: Sets the URL-encoded path to a custom GitLab installation to use for GitLab file operations.
   */
  gitlab?: string;
  /**
   * ID: Specifies the client ID of the custom GitLab application when using a custom GitLab instance.
   */
  'gitlab-id'?: string;
  /**
   * 0: Disables the GitLab integration.
   */
  gl?: (On1 | Off1) & string;
  /**
   * 1: Uses the chromeles mode with the grid enabled.
   */
  grid?: (On2 | Off2) & string;
  /**
   * 1: Hides the controls that allow you to change the current page in lightbox mode. **NEW `14.6.10`**
   */
  'hide-pages'?: (Off | On) & string;
  /**
   * hex: Specifies the highlight colour (with no leading `#`) for links in chromeless mode.
   */
  highlight?: string;
  /**
   * 1: If modified specifies a resource key, this is used to keep the modified state after Save is selected. **TBD `14.6.10`**
   */
  keepmodified?: (On2 | Off2) & string;
  /**
   * xy: Specifies the language of the user interface. Possible values for `xy` include  *i18n* ar, bs, ca, cs, da, de, el, en, es, et, fi, fil, fr, he, hu, id, it, ja, ko, ms, nl, no, pl, pt, pt-br, ro, ru, sr, sv, th, tr, uk, vi, zh, zh-tw
   */
  lang?: string;
  /**
   * 1: Adds layer control in chromeless mode
   */
  layers?: (On2 | Off2) & string;
  /**
   * 1: Specifies whether libraries should be enabled in embed mode. The default is disabled (0). **TBD `14.6.10`**
   */
  libraries?: (On2 | Off2) & string;
  /**
   * key1;key2;...;keyN: Specifies the current libraries. Possible keys are allied_telesis, android, archimate, archimate3, arrows2, atlassian, aws3, aws3d, aws4, azure, basic, bootstrap, bpmn, cabinets, cisco, cisco\_safe, citrix, clipart, dfd, eip, electrical, er, floorplan, flowchart, gcp2, general, gmdl, ibm, images, infographic, ios, lean\_mapping, mockups, mscae, network, office, pid, rack, signs, sitemap, sysml, uml, veeam and webicons.
   */
  libs?: string;
  /**
   * 1: Uses the lightbox in chromeless mode (larger zoom, no page visible, chromeless).
   */
  lightbox?: (On2 | Off2) & string;
  /**
   * 1: Uses device mode only.
   */
  local?: (On2 | Off2) & string;
  /**
   * 0: Disables MathJax support.
   */
  math?: (On2 | Off2) & string;
  /**
   * : Sets the font to use for math typesetting. Possible values are STIX-Web and TeX (default). **TBD `14.6.10`**
   */
  'math-font'?: string;
  /**
   * html: Sets the math typesetting output to `HTML-CSS`. **NEW `14.6.10`**
   */
  'math-output'?: {
    [k: string]: unknown;
  };
  /**
   * [google|onedrive|github|dropobox|device|browser]: Switches to the specified mode.
   */
  mode?: 'google' | 'onedrive' | 'github' | 'dropobox' | 'device' | 'browser';
  /**
   * 1: Enables folding in chromeless mode.
   */
  nav?: (On2 | Off2) & string;
  /**
   * 1: Displays no Exit button. Instead of using this URL parameter, you can specify this setting in the load message. **TBD `14.6.10`**
   */
  noExitBtn?: (On2 | Off2) & string;
  /**
   * 1: Displays a Save and Exit button instead of a Save button. Instead of using this URL parameter, you can specify this setting in the load message. If this is used, then saveAndExit is ignored. **TBD `14.6.10`**
   */
  noSaveBtn?: (On2 | Off2) & string;
  /**
   * 1: Uses the default filename for new files (used with the `url` parameter)
   */
  notitle?: (On2 | Off2) & string;
  /**
   * 1: Suppresses the warning before an unload for local files.
   */
  nowarn?: (On2 | Off2) & string;
  /**
   * 0: Disables the OneDrive integration.
   */
  od?: (On1 | Off1) & string;
  /**
   * [1|0]: Registers or unregisters the progressive web app (all remote storage locations are disabled).
   */
  offline?: (Off | On) & string;
  /**
   * prefix+ID: If location hash properties are not available, use this as a replacement. The browser will add the hash property to the URL immediately and not remove the URL parameter. Existing hash property takes precedence.
   */
  open?: string;
  /**
   * 1: Saves all Google Drive files as `application/vnd.jgraph.mxfile`.
   */
  'override-mime'?: (On2 | Off2) & string;
  /**
   * id1;id2;...;idN: Selects which plugins to load. See the list of available plugins.
   */
  p?: string;
  /**
   * index: Starts with the given page (the default is the first page, where the index for the first page is `0`)
   */
  page?: number;
  /**
   * ID: Starts with the given page ID (has precedence over the `page` URL parameter). To find the ID of the current page, right click on the canvas and select *Edit Data*. The ID is the first entry in this dialog.
   */
  'page-id'?: string;
  /**
   * 0/1: Disables/enables the Google file picker in dialogs.
   */
  picker?: (Off | On) & string;
  /**
   * 0: Does not load plugins.
   */
  plugins?: (On1 | Off1) & string;
  /**
   * `client=1`: Runs diagrams.net in client mode. This displays a normal UI and sends a `ready` message to the opener or parent when the page is loaded. After receiving a message containing XML or compressed XML, a local file is created. The file is then set to `modified` so the connection to the window can be closed after sending the initial XML. Whether the XML is loaded again after a page refresh, depends on the caller implementation. Use `proto=json` to use a custom protocol for special cases. **NEW `14.6.10`**
   */
  proto?: {
    [k: string]: unknown;
  };
  /**
   * 0: Sets the default `pageVisible` to `false`.
   */
  pv?: (On1 | Off1) & string;
  /**
   * 0: Disables the ability to (un)register the progressive web app (when installed it will stay installed). **NEW `14.6.10`**
   */
  pwa?: (On1 | Off1) & string;
  /**
   * 1: Returns a JSON structure with the diagram bounds in embed and client mode. This message is dispatched immediately after receiving the diagram XML. **TBD `14.6.10`**
   */
  returnbounds?: (On1 | Off1) & string;
  /**
   * id: Selects a specific revision of a Google Drive or Dropbox file.
   */
  rev?: string;
  /**
   * [0|1]: Disables/enables the rough sketch style (default is 1 for sketch theme and 0 for all other themes).
   */
  rough?: (Off | On) & string;
  /**
   * 1: Uses chromeless mode with Google Drive realtime enabled.
   */
  rt?: (On2 | Off2) & string;
  /**
   * 1: Starts with the ruler enabled.
   */
  ruler?: (On2 | Off2) & string;
  /**
   * local/remote: Enables/disables saving files locally (default is `local`). If this is disabled, then files are echoed via a servlet for a local save.
   */
  save?: 'local' | 'remote';
  /**
   * 1: Displays a Save and Exit button ne. Instead of using this URL parameter, you can specify this setting in the load message. **TBD `14.6.10`**
   */
  saveAndExit?: (On2 | Off2) & string;
  /**
   * 0: Starts with scrollbars disabled.
   */
  sb?: (On1 | Off1) & string;
  /**
   * value: Starts with given value in the search shapes box.  ## Modes **NEW `14.6.10`**
   */
  'search-shapes'?: string;
  /**
   * 1: Shows a Loading... spinner while waiting for the diagram data to load in embed mode. **TBD `14.6.10`**
   */
  spin?: (On2 | Off2) & string;
  /**
   * 0: Does not show the splash screen.
   */
  splash?: (On1 | Off1) & string;
  /**
   * 1: Disables all features that require external web services (such as PDF export).
   */
  stealth?: (On2 | Off2) & string;
  /**
   * device: Adds a device storage option for touch devices.
   */
  storage?: 'device';
  /**
   * 0: Suppresses the warning for old viewers in SVG export. **NEW `14.6.10`**
   */
  'svg-warning'?: (On1 | Off1) & string;
  /**
   * [none|manual|auto]: Enables/disables collaborative editing (default is `auto`).
   */
  sync?: 'none' | 'manual' | 'auto';
  /**
   * [auto|self|frame|blank]: Opens links in the same window or frame or in a blank window in chromeless mode (`auto` is the default which opens relative links and anchors in the same window in chromeless mode, and all links in a new window in editing mode).
   */
  target?: 'auto' | 'self' | 'frame' | 'blank';
  /**
   * name: If the \#U hash property is used but the URL does not contain a filename to determine binary mode. See the list of supported location hash properties.
   */
  'template-filename'?: string;
  /**
   * 0: Disables the creation of thumbnails in Drive.
   */
  thumb?: (On1 | Off1) & string;
  /**
   * title: Sets a new file title (used with the `create` and `url` parameters).
   */
  title?: string;
  /**
   * 0: Disables the toolbar in chromeless mode.
   */
  toolbar?: (On1 | Off1) & string;
  /**
   * 0: Disables the Trello integration.
   */
  tr?: (On1 | Off1) & string;
  ui?: UITheme;
  /**
   * url: *Deprecated* - Use U hash property `#U{uri_encoded_url}` to open a diagram from a URI-encoded URL, eg. https://app.diagrams.net/\#Uhttps%3A%2F%2Fraw.githubusercontent.com%2Fjgraph%2Fdrawio-diagrams%2Fmaster%2Fdiagrams%2Fschema.xml **NEW `14.6.10`**
   */
  url?: {
    [k: string]: unknown;
  };
  /**
   * `vars=json`: Uses a URI-encoded JSON string for global placeholders for use in labels and tooltips where enabled, e.g. `vars={"key":"value"}`.
   */
  vars?: string;
  /**
   * {"x": int, "y": int, "width": int, "height": int}: Sets the initial viewport when you specify file IDs. **NEW `14.6.10`**
   */
  viewbox?: {
    [k: string]: unknown;
  };
  /**
   * nocss: Disables the CSS zoom preview.
   */
  zoom?: string;
  [k: string]: unknown;
}
